//Standard 3D math operations
#include "3DMath.h"
#include <math.h>
#include <memory.h>

Matrix Matrix::operator * (Matrix& Mat2)
{
	Matrix RetMat;
	MatrixMultiply(&RetMat, (Matrix*)this, (Matrix*)&Mat2);
	return RetMat;
}

Matrix* MatrixMultiply(Matrix *MatrixOut, Matrix *Matrix1, Matrix *Matrix2)
{
	for(int i=0;i<4;i++)
		for(int p=0;p<4;p++)
		{
			MatrixOut->m[i][p]=0;
			for(int s=0;s<4;s++)
				MatrixOut->m[i][p] += Matrix1->m[i][s]*Matrix2->m[s][p];
		}
	return MatrixOut;
}

Vector4 MatrixVectorMultiply(Matrix *A, Vector3 *B)
{
	Vector4 ReturnVec;
	ReturnVec.w = A->_14*B->x+A->_24*B->y+A->_34*B->z+A->_44*1;
	ReturnVec.x = (A->_11*B->x+A->_21*B->y+A->_31*B->z+A->_41*1);
	ReturnVec.y = (A->_12*B->x+A->_22*B->y+A->_32*B->z+A->_42*1);
	ReturnVec.z = (A->_13*B->x+A->_23*B->y+A->_33*B->z+A->_43*1);

	return ReturnVec;
}

Vector3 MatrixVectorProject(Matrix *A, Vector3 *B)
{
	Vector4 ReturnVec;
	ReturnVec.w = A->_14*B->x+A->_24*B->y+A->_34*B->z+A->_44*1;
	ReturnVec.x = (A->_11*B->x+A->_21*B->y+A->_31*B->z+A->_41*1)/ReturnVec.w;
	ReturnVec.y = (A->_12*B->x+A->_22*B->y+A->_32*B->z+A->_42*1)/ReturnVec.w;
	ReturnVec.z = (A->_13*B->x+A->_23*B->y+A->_33*B->z+A->_43*1)/ReturnVec.w;

	return *(Vector3*)&ReturnVec;
}

float Vec3Dot(Vector3 *TheVec1, Vector3 *TheVec2)
{
	return TheVec1->x*TheVec2->x+TheVec1->y*TheVec2->y+TheVec1->z*TheVec2->z;
}

Vector3 NormalizeVec(Vector3 TheVec)

{
	float Mag=(float)sqrt(Vec3Dot(&TheVec, &TheVec));
	return Vector3(TheVec.x/Mag, TheVec.y/Mag, TheVec.z/Mag);
}

TwoDCoord GetTwoDCoord(Vertex *TheVert, Matrix* ProjectionMatrix, int Width, int Height)
{
	Vector3 FinalPoint;
	FinalPoint=MatrixVectorProject(ProjectionMatrix, TheVert);
	return TwoDCoord((int)((FinalPoint.x/2+.5)*Width), (int)((FinalPoint.y/-2+.5)*Height));
}

Matrix* MatrixLookAtLH(Matrix *pOut, Vector3 *pEye, Vector3 *pAt, Vector3 *pUp)
{

	Vector3 zaxis = NormalizeVec(*pAt-*pEye);
	Vector3 xaxis = NormalizeVec(*pUp*zaxis);
	Vector3 yaxis = zaxis*xaxis;
	float MatrixF[16]={xaxis.x,yaxis.x,zaxis.x,0,xaxis.y,yaxis.y,zaxis.y,0,xaxis.z,yaxis.z,zaxis.z,0,-Vec3Dot(&xaxis,pEye),-Vec3Dot(&yaxis,pEye),-Vec3Dot(&zaxis,pEye),1};

	memcpy(pOut, MatrixF, sizeof(Matrix));
	return pOut;
}

Matrix* MatrixPerspectiveFovLH(Matrix *pOut, float fovy, float Aspect, float zn, float zf)
{
	memset(pOut, 0, sizeof(Matrix));
	float YScale=1/(float)tan(fovy/2);
	pOut->_11=Aspect*YScale;
	pOut->_22=YScale;
	pOut->_33=zf/(zf-zn);
	pOut->_43=-zn*zf/(zf-zn);
	pOut->_34=1;
	return pOut;
}

Matrix* MatrixRotationX(Matrix *pOut, float Angle)
{
	memset(pOut, 0, sizeof(Matrix));
	pOut->_22=pOut->_33=(float)cos(Angle);
	pOut->_23=(float)sin(Angle);
	pOut->_32=-pOut->_23;
	pOut->_11=pOut->_44=1;
	return pOut;
}

Matrix* MatrixRotationY(Matrix *pOut, float Angle)
{
	memset(pOut, 0, sizeof(Matrix));
	pOut->_11=pOut->_33=(float)cos(Angle);
	pOut->_31=(float)sin(Angle);
	pOut->_13=-pOut->_31;
	pOut->_22=pOut->_44=1;
	return pOut;
}

Matrix* MatrixRotationZ(Matrix *pOut, float Angle)
{
	memset(pOut, 0, sizeof(Matrix));
	pOut->_11=pOut->_22=(float)cos(Angle);
	pOut->_12=(float)sin(Angle);
	pOut->_21=-pOut->_12;
	pOut->_33=pOut->_44=1;
	return pOut;
}